/*
  Copyright 2018-2020, Barcelona Supercomputing Center (BSC), Spain
  Copyright 2015-2020, Johannes Gutenberg Universitaet Mainz, Germany

  This software was partially supported by the
  EC H2020 funded project NEXTGenIO (Project ID: 671951, www.nextgenio.eu).

  This software was partially supported by the
  ADA-FS project under the SPPEXA project funded by the DFG.

  SPDX-License-Identifier: MIT
*/


#include <daemon/relocation/config_manager.hpp>
#include <global/hostsfile_util.hpp>

#include <iostream>

#include <daemon/daemon.hpp>
#include <daemon/handler/rpc_defs.hpp>


// #include <string>
// #include <spdlog/spdlog.h>

// #include <config.hpp>
#include <global/global_defs.hpp>
#include <global/hostsfile_util.hpp>
#include <global/rpc/rpc_types.hpp>


#include <fmt/format.h>
#include <cassert>

// margo
extern "C" {
// #include <abt.h>
// #include <mercury.h>
#include <margo.h>
}

// #include <daemon/classes/fs_data.hpp>
// #include <daemon/classes/rpc_data.hpp>

using namespace std;

namespace gkfs::relocation {


vector<pair<string, string>>
read_hosts_file() {
    string hostfile = GKFS_DATA->hosts_file();

    vector<pair<string, string>> hosts;
    try {
        hosts = gkfs::util::load_hostfile(hostfile);
    } catch(const exception& e) {
        auto emsg = fmt::format("Failed to load hosts file: {}", e.what());
        throw runtime_error(emsg);
    }

    if(hosts.empty()) {
        throw runtime_error(fmt::format("Hostfile empty: '{}'", hostfile));
    }

    GKFS_DATA->spdlogger()->info("Hosts pool size: {}", hosts.size());

    return hosts;
}

void
test_d2d_rpc() {
    auto hosts = read_hosts_file();

    cout << "number of hosts: " << hosts.size() << "\n";
    auto mid =
            margo_init(GKFS_DATA->bind_addr().c_str(), MARGO_CLIENT_MODE, 0, 0);
    assert(mid);


    hg_id_t relocation_start_id = MARGO_REGISTER(
            mid, gkfs::rpc::tag::relocation_start, rpc_relocation_start_in_t,
            rpc_err_out_t, rpc_srv_relocation_start);


    uint64_t host_counter = 0;
    for(auto host : hosts) {
        cout << fmt::format("Host {} bla {}\n", host.first, host.second);
        hg_addr_t host_addr;
        // TODO
        rpc_relocation_start_in_t in{};
        rpc_err_out_t out{};
        in.host_id = host_counter; // TODO is this consistent with client?
        in.host_size = hosts.size();
        auto ret = margo_addr_lookup(mid, host.second.c_str(), &host_addr);
        assert(ret == HG_SUCCESS);

        // let's do this sequential first
        hg_handle_t handle;
        ret = margo_create(mid, host_addr, relocation_start_id, &handle);
        assert(ret == HG_SUCCESS);

        ret = margo_forward(handle, &in); // blocking
        assert(ret == HG_SUCCESS);

        ret = margo_get_output(handle, &out);
        assert(ret == HG_SUCCESS);

        // TODO process output

        ret = margo_free_output(handle, &out);
        assert(ret == HG_SUCCESS);

        host_counter++;
    }
}

} // namespace gkfs::relocation